package main

import (
	"io/ioutil"
	"os"
	pathpkg "path"
	"path/filepath"
	"strings"
	"text/template"
)

// Templates contains site templates.
type Templates struct {
	tmpls map[string]*template.Template
	funcs template.FuncMap
}

// NewTemplates returns a new Templates with the default templates.
func NewTemplates() *Templates {
	t := &Templates{
		tmpls: map[string]*template.Template{},
	}
	return t
}

// Funcs sets the functions available to newly created templates.
func (t *Templates) Funcs(funcs template.FuncMap) {
	t.funcs = funcs
}

// LoadTemplate loads a template from the provided path and content.
func (t *Templates) LoadTemplate(path string, content []byte) {
	tmpl := template.New(path)
	tmpl.Funcs(t.funcs)
	template.Must(tmpl.Parse(string(content)))
	t.tmpls[path] = tmpl
}

// Load loads templates from the provided directory
func (t *Templates) Load(dir string) error {
	return filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err != nil {
			return err
		}
		if info.Mode().IsRegular() {
			b, err := ioutil.ReadFile(path)
			if err != nil {
				return err
			}
			// Remove directory from beginning of path
			path = strings.TrimPrefix(path, dir)
			t.LoadTemplate(path, b)
		}
		return nil
	})
}

// FindTemplate returns the template for the given path.
func (t *Templates) FindTemplate(path string, tmpl string) (*template.Template, bool) {
	tmplPath := pathpkg.Join(path, tmpl)
	if t, ok := t.tmpls[tmplPath]; ok {
		return t, true
	}
	if t, ok := t.tmpls[pathpkg.Join("/_default", tmpl)]; ok {
		return t, true
	}
	// Failed to find template
	return nil, false
}
